use reqwest::blocking as reqwest;
use std::collections::HashMap;
use std::convert::{TryFrom, From};
use rocket::http::Status;
use std::error::Error;

pub mod structs;
use structs::{Street, Date, PickupTime};


/// The base URL where their API starts
const BASE_URL: &str = "https://www.ivago.be/nl/particulier/afval/ophaling";
const SEARCH_URL: &str ="https://www.ivago.be/nl/particulier/autocomplete/garbage/streets";


impl From<Street> for String {
    fn from(street: Street) -> String {
        format!("{} ({})", street.name, street.city)
    }
}


/// Searches the Ivago API for streets in the given city
///
/// # Arguments
///
/// * `street` - name of the street
/// * `city` - city the street is in
// TODO find out how to do this async
pub fn search_streets(street_name: &String) -> Result<Vec<Street>, Box<dyn Error>> {
    let client = reqwest::Client::new();
    let response = client.get(SEARCH_URL)
                         .query(&[("q", street_name)])
                         .send()?;
    let data: Vec<HashMap<String, String>> = response.json()?;

    let mut output: Vec<Street> = Vec::new();

    // We iterate over every item and extract the needed data
    for map in data.iter() {
        if let Some(value) = map.get("value") {
            match Street::try_from(*value) {
                Ok(street) => output.push(street),
                Err(_) => continue,
            }
        }
    }

    Ok(output)
}


/// Return the known pickup times for the given street and/or city
///
/// # Arguments
///
/// * `street` - name of the street
/// * `city` - city the street is in
pub fn get_pickup_times(street: Street, number: u32) -> Result<Vec<PickupTime>, Box<dyn Error>> {
    // The client needs to store cookies for the requests to work
    let client = reqwest::Client::builder().cookie_store(true).build()?;

    // Create post data
    let form = [
        ("garbage_type", ""),
        ("ivago_street", String::from(street).as_str()),
        ("number", format!("{}", number).as_str()),
        ("form_id", "garbage_address_form"),
    ];

    // This request just serves to populate the cookies
    client.post(BASE_URL)
          .form(&form)
          .send()?;

    let params = [
        ("_format", "json"),
        ("type", ""),
        
    ]

r2 = s.get("https://www.ivago.be/nl/particulier/garbage/pick-up/pickups?",
                 params={
                     "_format": "json",
                     "type": "",
                     "start": "1622332800",
                     "end": "163861328100"
                 }
}
